"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var _a, _b;
var abstract_1 = require("./abstract");
var promise_1 = require("./utils/promise");
var InternalCall = Symbol('RPCClient internal call');
var InternalPost = Symbol('RPCClient internal post');
var RPCWithTransfer = /** @class */ (function () {
    function RPCWithTransfer(client, transfers) {
        var _this = this;
        this.client = client;
        this.transfers = transfers;
        this.call = function (method) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var _a;
            return (_a = _this.client)[InternalCall].apply(_a, [method, _this.transfers].concat(args));
        };
        this.post = function (method) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var _a;
            (_a = _this.client)[InternalPost].apply(_a, [method, _this.transfers].concat(args));
        };
    }
    return RPCWithTransfer;
}());
exports.RPCWithTransfer = RPCWithTransfer;
var RPCClient = /** @class */ (function (_super) {
    __extends(RPCClient, _super);
    function RPCClient(worker) {
        var _this = _super.call(this, worker) || this;
        _this.idSerial = 0;
        _this.resolvers = {};
        _this.call = function (method) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            return _this[InternalCall].apply(_this, [method, undefined].concat(args));
        };
        _this[_a] = function (method, transfer) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var request = {
                id: _this.idSerial++,
                type: method,
                args: args
            };
            var holder = new promise_1.ValueResolver();
            _this.resolvers[request.id] = holder;
            return holder.promise(function () {
                _this.channel.postMessage(request, transfer);
            });
        };
        _this.post = function (method) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            _this[InternalPost].apply(_this, [method, undefined].concat(args));
        };
        _this[_b] = function (method, transfer) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var request = {
                id: _this.idSerial++,
                type: method,
                args: args,
            };
            _this.channel.postMessage(request, transfer);
        };
        _this.onMessage = function (message) {
            var response = message.data;
            // Channel can be used as duplex.
            if (message.data.type) {
                return;
            }
            var resolver = _this.resolvers[response.id];
            delete _this.resolvers[response.id];
            if (!resolver) {
                return _this.fireError(new Error("No resolver for " + response.id));
            }
            try {
                if (response.error) {
                    return resolver.reject(response.error);
                }
                else {
                    return resolver.resolve(response.result);
                }
            }
            catch (error) {
                return _this.fireError(new Error(error.message));
            }
        };
        _this.channel.addEventListener('message', _this.onMessage);
        return _this;
    }
    RPCClient.prototype.withTransfer = function () {
        var transfers = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            transfers[_i] = arguments[_i];
        }
        return new RPCWithTransfer(this, transfers);
    };
    return RPCClient;
}(abstract_1.AbstractRPC));
_a = InternalCall, _b = InternalPost;
exports.RPCClient = RPCClient;
